@@ -16,7 +16,7 @@ from rich.theme import Theme
 from natsort import natsorted, ns
 import regex
 
-VERSION = "1.8.0"
+VERSION = "1.8.3"
 MESSAGE_INDENT = "    "
 UNICODE_BOM = "\ufeff"
 NO_BREAK_SPACE = "\u00a0"
@@ -27,8 +27,6 @@ SHY_HYPHEN = "\u00ad"
 FUNCTION_APPLICATION = "\u2061"
 NO_BREAK_HYPHEN = "\u2011"
 IGNORED_FILENAMES = ["colophon.xhtml", "titlepage.xhtml", "imprint.xhtml", "uncopyright.xhtml", "halftitle.xhtml", "toc.xhtml", "loi.xhtml"]
-# The `re` namespace enables regex functions in xpaths in lxml
-XHTML_NAMESPACES = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "z3998": "http://www.daisy.org/z3998/2012/vocab/structure/", "se": "https://standardebooks.org/vocab/1.0", "dc": "http://purl.org/dc/elements/1.1/", "opf": "http://www.idpf.org/2007/opf", "container": "urn:oasis:names:tc:opendocument:xmlns:container", "m": "http://www.w3.org/1998/Math/MathML", "re": "http://exslt.org/regular-expressions"}
 SELECTORS_TO_SIMPLIFY = [":first-child", ":only-child", ":last-child", ":nth-child", ":nth-last-child", ":first-of-type", ":only-of-type", ":last-of-type", ":nth-of-type", ":nth-last-of-type"]
 MESSAGE_TYPE_WARNING = 1
 MESSAGE_TYPE_ERROR = 2
@@ -224,10 +222,15 @@ def get_target_filenames(targets: list, allowed_extensions: tuple, ignored_filen
 
 	return natsorted(list(target_xhtml_filenames), key=lambda x: str(x.name), alg=ns.PATH)
 
-def is_called_from_parallel() -> bool:
+def is_called_from_parallel(return_none=True) -> Union[bool,None]:
 	"""
 	Decide if we're being called from GNU parallel.
 	This is good to know in case we want to tweak some output.
+
+	This is almost always passed directly to the force_terminal option of rich.console(),
+	meaning that `None` means "guess terminal status" and `False` means "no colors at all".
+	We typically want to guess, so this returns None by default if not called from Parallel.
+	To return false in that case, pass return_none=False
 	"""
 
 	import psutil # pylint: disable=import-outside-toplevel
@@ -240,4 +243,4 @@ def is_called_from_parallel() -> bool:
 		# If we can't figure it out, don't worry about it
 		pass
 
-	return False
+	return None if return_none else False
