@@ -15,6 +15,7 @@ import git
 import importlib_resources
 import regex
 import requests
+from rich.console import Console
 from ftfy import fix_text
 from lxml import etree
 
@@ -655,8 +656,9 @@ def _create_draft(args: Namespace):
 	if args.pg_url and pg_ebook_html:
 		try:
 			dom = etree.parse(StringIO(regex.sub(r"encoding=\".+?\"", "", pg_ebook_html)), parser)
+			namespaces = {"re": "http://exslt.org/regular-expressions"}
 
-			for node in dom.xpath("//*[re:test(text(), '\\*\\*\\*\\s*Produced by.+')]", namespaces=se.XHTML_NAMESPACES):
+			for node in dom.xpath("//*[re:test(text(), '\\*\\*\\*\\s*Produced by.+')]", namespaces=namespaces):
 				producers_text = regex.sub(r"^<[^>]+?>", "", etree.tostring(node, encoding=str, with_tail=False))
 				producers_text = regex.sub(r"<[^>]+?>$", "", producers_text)
 
@@ -668,10 +670,10 @@ def _create_draft(args: Namespace):
 				producers_text = producers_text.replace(" and the Online", " and The Online")
 				producers_text = producers_text.replace(", and ", ", ").strip()
 
-				pg_producers = regex.split(',|;', producers_text)
+				pg_producers = [producer.strip() for producer in regex.split(',|;', producers_text)]
 
 			# Try to strip out the PG header
-			for node in dom.xpath("//*[re:test(text(), '\\*\\*\\*\\s*START OF THIS')]", namespaces=se.XHTML_NAMESPACES):
+			for node in dom.xpath("//*[re:test(text(), '\\*\\*\\*\\s*START OF THIS')]", namespaces=namespaces):
 				for sibling_node in node.xpath("./preceding-sibling::*"):
 					easy_node = se.easy_xml.EasyXmlElement(sibling_node)
 					easy_node.remove()
@@ -680,7 +682,7 @@ def _create_draft(args: Namespace):
 				easy_node.remove()
 
 			# Try to strip out the PG license footer
-			for node in dom.xpath("//*[re:test(text(), 'End of (the )?Project Gutenberg')]", namespaces=se.XHTML_NAMESPACES):
+			for node in dom.xpath("//*[re:test(text(), 'End of (the )?Project Gutenberg')]", namespaces=namespaces):
 				for sibling_node in node.xpath("./following-sibling::*"):
 					easy_node = se.easy_xml.EasyXmlElement(sibling_node)
 					easy_node.remove()
@@ -769,11 +771,17 @@ def _create_draft(args: Namespace):
 
 		colophon_xhtml = colophon_xhtml.replace("SE_IDENTIFIER", identifier)
 		colophon_xhtml = colophon_xhtml.replace("TITLE", title)
-		colophon_xhtml = colophon_xhtml.replace("<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", _generate_contributor_string(authors, True))
+
+		contributor_string =  _generate_contributor_string(authors, True)
+
+		if contributor_string == "":
+			colophon_xhtml = colophon_xhtml.replace(" by<br/>\n\t\t\t<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
+		else:
+			colophon_xhtml = colophon_xhtml.replace("<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
 
 		if translators:
 			translator_block = f"It was translated from ORIGINAL_LANGUAGE in TRANSLATION_YEAR by<br/>\n\t\t\t{_generate_contributor_string(translators, True)}.</p>"
-			colophon_xhtml = colophon_xhtml.replace("<p>This ebook was produced for the<br/>", f"{translator_block}\n\t\t\t<p>This ebook was produced for the<br/>")
+			colophon_xhtml = colophon_xhtml.replace("</p>\n\t\t\t<p>This ebook was produced for the<br/>", f"<br/>\n\t\t\t{translator_block}\n\t\t\t<p>This ebook was produced for the<br/>")
 
 		if args.pg_url:
 			colophon_xhtml = colophon_xhtml.replace("PG_URL", args.pg_url)
@@ -842,13 +850,13 @@ def _create_draft(args: Namespace):
 			translators_xml = _generate_metadata_contributor_xml(translators, "translator")
 			metadata_xml = regex.sub(r"<dc:contributor id=\"translator\">.+?scheme=\"marc:relators\">trl</meta>", translators_xml, metadata_xml, flags=regex.DOTALL)
 		else:
-			metadata_xml = regex.sub(r"<dc:contributor id=\"translator\">.+?scheme=\"marc:relators\">trl</meta>", "", metadata_xml, flags=regex.DOTALL)
+			metadata_xml = regex.sub(r"<dc:contributor id=\"translator\">.+?scheme=\"marc:relators\">trl</meta>\n\t\t", "", metadata_xml, flags=regex.DOTALL)
 
 		if illustrators:
 			illustrators_xml = _generate_metadata_contributor_xml(illustrators, "illustrator")
 			metadata_xml = regex.sub(r"<dc:contributor id=\"illustrator\">.+?scheme=\"marc:relators\">ill</meta>", illustrators_xml, metadata_xml, flags=regex.DOTALL)
 		else:
-			metadata_xml = regex.sub(r"<dc:contributor id=\"illustrator\">.+?scheme=\"marc:relators\">ill</meta>", "", metadata_xml, flags=regex.DOTALL)
+			metadata_xml = regex.sub(r"<dc:contributor id=\"illustrator\">.+?scheme=\"marc:relators\">ill</meta>\n\t\t", "", metadata_xml, flags=regex.DOTALL)
 
 		if args.pg_url:
 			if pg_subjects:
@@ -920,6 +928,13 @@ def create_draft() -> int:
 		return se.InvalidArgumentsException.code
 
 	try:
+		# Before we continue, confirm that there isn't a subtitle passed in with the title
+		if ":" in args.title:
+			console = Console(highlight=False, theme=se.RICH_THEME) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
+			console.print("Titles should not include a subtitle, as subtitles are separate metadata elements in [path]content.opf[/]. Are you sure you want to continue? \\[y/N]")
+			if input().lower() not in {"yes", "y"}:
+				return se.InvalidInputException.code
+
 		_create_draft(args)
 	except se.SeException as ex:
 		se.print_error(ex)
