@@ -9,22 +9,13 @@ import unicodedata
 
 import regex
 from lxml import cssselect, etree
-import se
+from cssselect import parser
 
+import se
+import se.css
 
 CSS_SELECTOR_CACHE: Dict[str, cssselect.CSSSelector] = {}
-
-def css_selector(selector: str) -> cssselect.CSSSelector:
-	"""
-	Create a CSS selector for the given selector string. Return a cached CSS selector if
-	one already exists.
-	"""
-
-	sel = CSS_SELECTOR_CACHE.get(selector)
-	if not sel:
-		sel = cssselect.CSSSelector(selector, translator="xhtml", namespaces=se.XHTML_NAMESPACES)
-		CSS_SELECTOR_CACHE[selector] = sel
-	return sel
+CSS_RULES_CACHE: Dict[str, List[se.css.CssRule]] = {}
 
 def escape_xpath(string: str) -> str:
 	"""
@@ -52,14 +43,24 @@ class EasyXmlTree:
 	"""
 
 	def __init__(self, xml_string: str):
+		self.namespaces = {"re": "http://exslt.org/regular-expressions"} # Enable regular expressions in xpath
 		self.etree = etree.fromstring(str.encode(xml_string))
+		self.is_css_applied = False
 
-	def css_select(self, selector: str) -> Union[str, list, None]:
+	def css_select(self, selector: str):
 		"""
 		Shortcut to select elements based on CSS selector.
 		"""
 
-		return self.xpath(css_selector(selector).path)
+		try:
+			sel = CSS_SELECTOR_CACHE.get(selector)
+			if not sel:
+				sel = cssselect.CSSSelector(selector, translator="xhtml", namespaces={"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops"})
+				CSS_SELECTOR_CACHE[selector] = sel
+
+			return self.xpath(sel.path)
+		except parser.SelectorSyntaxError as ex:
+			raise se.InvalidCssException(f"Invalid selector: [css]{selector}[/]") from ex
 
 	def xpath(self, selector: str, return_string: bool = False):
 		"""
@@ -73,11 +74,11 @@ class EasyXmlTree:
 
 		result: List[Union[str, EasyXmlElement]] = []
 
-		for element in self.etree.xpath(selector, namespaces=se.XHTML_NAMESPACES):
+		for element in self.etree.xpath(selector, namespaces=self.namespaces):
 			if isinstance(element, str):
 				result.append(element)
 			else:
-				result.append(EasyXmlElement(element))
+				result.append(EasyXmlElement(element, self.namespaces))
 
 		if return_string and result:
 			return str(result[0])
@@ -86,11 +87,73 @@ class EasyXmlTree:
 
 		return result
 
+	@staticmethod
+	def _apply_css_declaration_to_node(node, declaration: se.css.CssDeclaration, specificity_number: int):
+		if declaration.applies_to == "all" or (declaration.applies_to == "block" and node.tag in se.css.CSS_BLOCK_ELEMENTS):
+			existing_specificity = node.get_attr(f"data-css-{declaration.name}-specificity") or 0
+
+			if declaration.important:
+				specificity_number = specificity_number + 1000
+
+			if int(existing_specificity) <= specificity_number:
+				node.set_attr(f"data-css-{declaration.name}", declaration.value)
+				node.set_attr(f"data-css-{declaration.name}-specificity", str(specificity_number))
+
+	def apply_css(self, css: str, filename: str=None):
+		"""
+		Apply a CSS stylesheet to an XHTML tree.
+		The application is naive and should not be expected to be browser-grade.
+		CSS properties on specific elements can be returned using EasyXmlElement.get_css_property()
+
+		With filename, save the resulting rules in a cache to prevent having to re-parse
+		the same stylesheet over and over.
+
+		Currently this does not support rules/declarations in @ blocks like @supports.
+
+		For example,
+
+		for node in dom.xpath("//em")"
+			print(node.get_css_property("font-style"))
+		"""
+		self.is_css_applied = True
+
+		if filename and filename in CSS_RULES_CACHE:
+			rules = CSS_RULES_CACHE[filename]
+		else:
+			rules = se.css.parse_rules(css)
+			if filename:
+				CSS_RULES_CACHE[filename] = rules
+
+		# We've parsed the CSS, now apply it to the DOM tree
+		for rule in rules:
+			try:
+				for node in self.css_select(rule.selector):
+					for declaration in rule.declarations:
+						self._apply_css_declaration_to_node(node, declaration, rule.specificity_number)
+
+						# If the property is inherited, apply it to its descendants
+						# However inherited properties get 0 specificity, because they can be overriden
+						if declaration.inherited:
+							for child in node.xpath(".//*"):
+								self._apply_css_declaration_to_node(child, declaration, 0)
+
+			except cssselect.ExpressionError:
+				# This gets thrown on some selectors not yet implemented by lxml, like *:first-of-type
+				pass
+
 	def to_string(self) -> str:
 		"""
 		Serialize the tree to a string.
 		"""
 
+		# If we applied a CSS file to this tree, remove the special attributes we
+		# added before printing it out.
+		if self.is_css_applied:
+			for node in self.xpath("//@*[starts-with(name(), 'data-css-')]/parent::*"):
+				for attr in node.lxml_element.attrib:
+					if attr.startswith("data-css-"):
+						node.remove_attr(attr)
+
 		xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + etree.tostring(self.etree, encoding="unicode") + "\n"
 
 		# Normalize unicode characters
@@ -98,6 +161,28 @@ class EasyXmlTree:
 
 		return xml
 
+class EasyContainerTree(EasyXmlTree):
+	"""
+	Wrapper for the container namespace.
+	"""
+
+	def __init__(self, xml_string: str):
+		# We have to remove the default namespace declaration from our document, otherwise
+		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
+
+		super().__init__(xml_string.replace(" xmlns=\"urn:oasis:names:tc:opendocument:xmlns:container\"", ""))
+
+	def to_string(self) -> str:
+		"""
+		Serialize the tree to a string.
+		"""
+
+		xml = EasyXmlTree.to_string(self)
+
+		xml = regex.sub(r"<container(?!:)", "<container xmlns=\"urn:oasis:names:tc:opendocument:xmlns:container\"", xml)
+
+		return xml
+
 class EasyXhtmlTree(EasyXmlTree):
 	"""
 	Wrapper for the XHTML namespace.
@@ -107,7 +192,9 @@ class EasyXhtmlTree(EasyXmlTree):
 		# We have to remove the default namespace declaration from our document, otherwise
 		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 
-		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", ""))
+		super().__init__(xml_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", ""))
+
+		self.namespaces = {**self.namespaces, **{"epub": "http://www.idpf.org/2007/ops", "m": "http://www.w3.org/1998/Math/MathML"}}
 
 	def to_string(self) -> str:
 		"""
@@ -129,7 +216,9 @@ class EasySvgTree(EasyXmlTree):
 		# We have to remove the default namespace declaration from our document, otherwise
 		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 
-		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/2000/svg\"", ""))
+		super().__init__(xml_string.replace(" xmlns=\"http://www.w3.org/2000/svg\"", ""))
+
+		self.namespaces = {**self.namespaces, **{"xlink": "http://www.w3.org/1999/xlink"}}
 
 	def to_string(self) -> str:
 		"""
@@ -151,7 +240,9 @@ class EasyOpfTree(EasyXmlTree):
 		# We have to remove the default namespace declaration from our document, otherwise
 		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 
-		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.idpf.org/2007/opf\"", ""))
+		super().__init__(xml_string.replace(" xmlns=\"http://www.idpf.org/2007/opf\"", ""))
+
+		self.namespaces = {**self.namespaces, **{"dc": "http://purl.org/dc/elements/1.1/"}}
 
 	def to_string(self) -> str:
 		"""
@@ -169,7 +260,8 @@ class EasyXmlElement:
 	Represents an lxml element.
 	"""
 
-	def __init__(self, lxml_element):
+	def __init__(self, lxml_element, namespaces=None):
+		self.namespaces = namespaces
 		self.lxml_element = lxml_element
 
 	def to_tag_string(self) -> str:
@@ -184,7 +276,9 @@ class EasyXmlElement:
 		attrs = ""
 
 		for name, value in self.lxml_element.items():
-			attrs += f" {name}=\"{value}\""
+			# Exclude applied CSS
+			if not name.startswith("data-css-"):
+				attrs += f" {name}=\"{value}\""
 
 		attrs = attrs.replace("{http://www.idpf.org/2007/ops}", "epub:")
 		attrs = attrs.replace("{http://www.w3.org/XML/1998/namespace}", "xml:")
@@ -200,7 +294,45 @@ class EasyXmlElement:
 		`<p>Hello there, <abbr>Mr.</abbr> Smith!</p>` -> `<p>Hello there, <abbr>Mr.</abbr> Smith!</p>`
 		"""
 
-		return regex.sub(r" xmlns(:[\p{Letter}]+?)?=\"[^\"]+?\"", "", etree.tostring(self.lxml_element, encoding=str, with_tail=False))
+		value = etree.tostring(self.lxml_element, encoding=str, with_tail=False)
+
+		# Remove namespaces
+		value = regex.sub(r" xmlns(:[\p{Letter}]+?)?=\"[^\"]+?\"", "", value)
+
+		# Remove applied CSS
+		value = regex.sub(r" data-css-[a-z\-]+?=\"[^\"]*?\"", "", value)
+
+		return value
+
+	def get_css(self) -> Dict:
+		"""
+		Return a dict of CSS properties applied to this node.
+		"""
+
+		output = {}
+		for attr, value in self.lxml_element.attrib.items():
+			if attr.startswith("data-css-"):
+				output[attr.replace("data-css-", "")] = value
+
+		return output
+
+	def get_css_property(self, property_name: str):
+		"""
+		Return the applied CSS value for the given property name, like `border-color`,
+		or None if it does not exist.
+		"""
+
+		if f"data-css-{property_name}" in self.lxml_element.attrib:
+			return self.lxml_element.attrib[f"data-css-{property_name}"]
+
+		return None
+
+	def remove_attr(self, attribute: str):
+		"""
+		Remove an attribute from this node.
+		"""
+
+		etree.strip_attributes(self.lxml_element, attribute)
 
 	def get_attr(self, attribute: str) -> str:
 		"""
@@ -228,11 +360,11 @@ class EasyXmlElement:
 
 		result: List[Union[str, EasyXmlElement]] = []
 
-		for element in self.lxml_element.xpath(selector, namespaces=se.XHTML_NAMESPACES):
+		for element in self.lxml_element.xpath(selector, namespaces=self.namespaces):
 			if isinstance(element, str):
 				result.append(element)
 			else:
-				result.append(EasyXmlElement(element))
+				result.append(EasyXmlElement(element, self.namespaces))
 
 		if return_string and result:
 			return str(result[0])
@@ -359,7 +491,7 @@ class EasyXmlElement:
 		Return an EasyXmlElement representing this node's parent node
 		"""
 
-		return EasyXmlElement(self.lxml_element.getparent())
+		return EasyXmlElement(self.lxml_element.getparent(), self.namespaces)
 
 	@property
 	def text(self) -> str:
