@@ -129,7 +129,7 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"""([^\p{Letter}>\"])([vxVX])(\b|st\b|nd\b|rd\b|th\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
 	# We can assume a lowercase i is always a Roman numeral
-	xhtml = regex.sub(r"""([^\p{Letter}<>/\"])(i)(\b|st\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
+	xhtml = regex.sub(r"""([^\p{Letter}<>/\"])i\b""", r"""\1<span epub:type="z3998:roman">i</span>""", xhtml)
 
 	# Fix X-ray
 	xhtml = regex.sub(r"""<span epub:type="z3998:roman">([Xx])</span>-ray""", r"""\1-ray""", xhtml)
@@ -625,6 +625,8 @@ def format_xhtml(xhtml: str) -> str:
 	A string of pretty-printed XHTML.
 	"""
 
+	namespaces = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "re": "http://exslt.org/regular-expressions"} # re enables regular expressions in xpath
+
 	# Epub3 doesn't allow named entities, so convert them to their unicode equivalents
 	# But, don't unescape the content.opf long-description accidentally
 	xhtml = regex.sub(r"&#?\w+;", _replace_character_references, xhtml)
@@ -644,13 +646,26 @@ def format_xhtml(xhtml: str) -> str:
 		raise se.InvalidXhtmlException(f"Couldn’t parse XHTML file. Exception: {ex}")
 
 	# Lowercase attribute names
-	for node in tree.xpath("//*[attribute::*[re:test(local-name(), '[A-Z]')]]", namespaces=se.XHTML_NAMESPACES):
+	for node in tree.xpath("//*[attribute::*[re:test(local-name(), '[A-Z]')]]", namespaces=namespaces):
 		for key, value in node.items(): # Iterate over attributes
 			node.attrib.pop(key) # Remove the attribute
 			node.attrib[key.lower()] = value # Re-add the attribute, lowercased
 
+	# Sort classes alphabetically, except the "eoc" class always comes last
+	for node in tree.xpath("//*[re:test(@class, '\\s')]", namespaces=namespaces):
+		# Sort class elements
+		classes = regex.split(r"\s+", node.get("class"))
+		classes = sorted(classes, key=str.lower)
+
+		# Move eoc to the end, if it exists
+		if "eoc" in classes:
+			classes += [classes.pop(classes.index("eoc"))]
+
+		# Set the new class value
+		node.set("class", " ".join(classes))
+
 	# Lowercase tag names
-	for node in tree.xpath("//*[re:test(local-name(), '[A-Z]')]", namespaces=se.XHTML_NAMESPACES):
+	for node in tree.xpath("//*[re:test(local-name(), '[A-Z]')]", namespaces=namespaces):
 		node.tag = node.tag.lower()
 
 	# Format <style> elements
@@ -1057,8 +1072,8 @@ def titlecase(text: str) -> str:
 	# Lowercase "the" if preceded by "vs."
 	text = regex.sub(r"(?:vs\.) The\b", "vs. the", text)
 
-	# Lowercase "de", "von", "van", "le", as in "Charles de Gaulle", "Werner von Braun", etc., and if not the first word and not preceded by an &ldquo;
-	text = regex.sub(r"(?<!^|“)\b(Von|Van|Le|Des|De La|De)\b", lambda result: result.group(1).lower(), text)
+	# Lowercase "de", "von", "van", "le", "du" as in "Charles de Gaulle", "Werner von Braun", etc., and if not the first word and not preceded by an &ldquo;
+	text = regex.sub(r"(?<!^|“)\b(Von|Van|Le|Des|De La|De|Du)\b", lambda result: result.group(1).lower(), text)
 
 	# Uppercase word following "Or,", since it is probably a subtitle
 	text = regex.sub(r"\bOr, ([\p{Lowercase_Letter}])", lambda result: "Or, " + result.group(1).upper(), text)
@@ -1202,6 +1217,7 @@ def simplify_css(css: str) -> str:
 
 	# Replace some more poorly-supported CSS attributes
 	css = css.replace("all-small-caps;", "small-caps;\n\ttext-transform: lowercase;")
+	css = regex.sub(r"text-align:\s*initial\s*;", "text-align: left;", css)
 
 	# Replace CSS namespace selectors with classes
 	# For example, p[epub|type~="z3998:salutation"] becomes p.epub-type-z3998-salutation
