@@ -209,7 +209,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 					xhtml = file.read().replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")
 					processed_xhtml = xhtml
 					try:
-						tree = etree.fromstring(str.encode(xhtml))
+						file_dom = se.easy_xml.EasyXhtmlTree(xhtml)
 					except Exception as ex:
 						raise se.InvalidXhtmlException(f"Error parsing XHTML file: [path][link=file://{filename}]{filename}[/][/]. Exception: {ex}")
 
@@ -226,13 +226,12 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 									replacement_class = split_selector[1].replace(":", "").replace("(", "-").replace("n-", "n-minus-").replace("n+", "n-plus-").replace(")", "")
 									selector = selector.replace(split_selector[1], "." + replacement_class, 1)
-									sel = se.easy_xml.css_selector(target_element_selector)
-									for element in tree.xpath(sel.path, namespaces=se.XHTML_NAMESPACES):
-										current_class = element.get("class", "")
+									for element in file_dom.css_select(target_element_selector):
+										current_class = element.get_attr("class") or ""
 
 										if replacement_class not in current_class:
 											current_class = f"{current_class} {replacement_class}".strip()
-											element.set("class", current_class)
+											element.set_attr("class", current_class)
 
 						except lxml.cssselect.ExpressionError:
 							# This gets thrown if we use pseudo-elements, which lxml doesn't support
@@ -243,63 +242,58 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						# We've already replaced attribute/namespace selectors with classes in the CSS, now add those classes to the matching elements
 						if "[epub|type" in selector:
 							for namespace_selector in regex.findall(r"\[epub\|type\~\=\"[^\"]*?\"\]", selector):
-								sel = se.easy_xml.css_selector(namespace_selector)
 
-								for element in tree.xpath(sel.path, namespaces=se.XHTML_NAMESPACES):
+								for element in file_dom.css_select(namespace_selector):
 									new_class = regex.sub(r"^\.", "", se.formatting.namespace_to_class(namespace_selector))
-									current_class = element.get("class", "")
+									current_class = element.get_attr("class") or ""
 
 									if new_class not in current_class:
 										current_class = f"{current_class} {new_class}".strip()
-										element.set("class", current_class)
+										element.set_attr("class", current_class)
 
-					processed_xhtml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" + etree.tostring(tree, encoding=str, pretty_print=True)
+					processed_xhtml = file_dom.to_string()
 
 					# We do this round in a second pass because if we modify the tree like this, it screws up how lxml does processing later.
 					# If it's all done in one pass, we wind up in a race condition where some elements are fixed and some not
-					tree = etree.fromstring(str.encode(processed_xhtml))
+					file_dom = se.easy_xml.EasyXhtmlTree(processed_xhtml)
 
 					for selector in selectors:
-						try:
-							sel = se.easy_xml.css_selector(selector)
-						except lxml.cssselect.ExpressionError:
-							# This gets thrown if we use pseudo-elements, which lxml doesn't support
-							continue
-						except lxml.cssselect.SelectorSyntaxError as ex:
-							raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: [css]{selector}[/]. Exception: {ex}")
-
-						# Convert <abbr> to <span>
 						if "abbr" in selector:
-							for element in tree.xpath(sel.path, namespaces=se.XHTML_NAMESPACES):
-								# Why would you want the tail to output by default?!?
-								raw_string = etree.tostring(element, encoding=str, with_tail=False)
+							try:
+								# Convert <abbr> to <span>
+								for element in file_dom.css_select(selector):
+									# Why would you want the tail to output by default?!?
+									raw_string = etree.tostring(element.lxml_element, encoding=str, with_tail=False)
 
-								# lxml--crap as usual--includes a bunch of namespace information in every element we print.
-								# Remove it here.
-								raw_string = raw_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")
-								raw_string = raw_string.replace(" xmlns:epub=\"http://www.idpf.org/2007/ops\"", "")
-								raw_string = raw_string.replace(" xmlns:m=\"http://www.w3.org/1998/Math/MathML\"", "")
+									# lxml--crap as usual--includes a bunch of namespace information in every element we print.
+									# Remove it here.
+									raw_string = raw_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")
+									raw_string = raw_string.replace(" xmlns:epub=\"http://www.idpf.org/2007/ops\"", "")
+									raw_string = raw_string.replace(" xmlns:m=\"http://www.w3.org/1998/Math/MathML\"", "")
 
-								# Now lxml doesn't let us modify the tree, so we just do a straight up regex replace to turn this into a span
-								processed_string = raw_string.replace("<abbr", "<span")
-								processed_string = processed_string.replace("</abbr", "</span")
+									# Now lxml doesn't let us modify the tree, so we just do a straight up regex replace to turn this into a span
+									processed_string = raw_string.replace("<abbr", "<span")
+									processed_string = processed_string.replace("</abbr", "</span")
 
-								# Now we have a nice, fixed string.  But, since lxml can't replace elements, we write it ourselves.
-								processed_xhtml = processed_xhtml.replace(raw_string, processed_string)
+									# Now we have a nice, fixed string.  But, since lxml can't replace elements, we write it ourselves.
+									processed_xhtml = processed_xhtml.replace(raw_string, processed_string)
 
-								tree = etree.fromstring(str.encode(processed_xhtml))
+									file_dom = se.easy_xml.EasyXhtmlTree(processed_xhtml)
+
+							except lxml.cssselect.ExpressionError:
+								# This gets thrown if we use pseudo-elements, which lxml doesn't support
+								continue
+							except lxml.cssselect.SelectorSyntaxError as ex:
+								raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: [css]{selector}[/]. Exception: {ex}")
 
 					# Now we just remove all stray abbr tags that were not styled by CSS
 					processed_xhtml = regex.sub(r"</?abbr[^>]*?>", "", processed_xhtml)
 
-					# Remove datetime="" attribute in <time> tags, which is not always understood by epubcheck
-					processed_xhtml = regex.sub(r" datetime=\"[^\"]+?\"", "", processed_xhtml)
-
-					tree = etree.fromstring(str.encode(processed_xhtml))
+					file_dom = se.easy_xml.EasyXhtmlTree(processed_xhtml)
 
 					if processed_xhtml != xhtml:
 						file.seek(0)
-						file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" + etree.tostring(tree, encoding=str, pretty_print=True).replace("<html", "<html xmlns=\"http://www.w3.org/1999/xhtml\""))
+						file.write(file_dom.to_string())
 						file.truncate()
 
 		# Done simplifying CSS and tags!
@@ -342,11 +336,6 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
 		metadata_xml = metadata_xml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
 
-		# Output the modified content.opf so that we can build the kobo book before making more epub2 compatibility hacks
-		with open(work_epub_root_directory / "epub" / "content.opf", "w", encoding="utf-8") as file:
-			file.write(metadata_xml)
-			file.truncate()
-
 		# Recurse over xhtml files to make some compatibility replacements
 		for root, _, filenames in os.walk(work_epub_root_directory):
 			for filename_string in filenames:
@@ -472,6 +461,9 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						for role in ARIA_ROLES:
 							processed_xhtml = regex.sub(fr"(epub:type=\"[^\"]*?{role}[^\"]*?\")", f"\\1 role=\"doc-{role}\"", processed_xhtml)
 
+						# We may have added multiple `role` attributes. epub doesn't allow more than one, so drop any others.
+						processed_xhtml = regex.sub(r"role=\"([^\"]+?)\"( role=\"([^\"]+?)\")+", "role=\"\\1\"", processed_xhtml)
+
 						# Some ARIA roles can't apply to some elements.
 						# For example, epilogue can't apply to <article>
 						processed_xhtml = regex.sub(r"<article ([^>]*?)role=\"doc-epilogue\"", "<article \\1", processed_xhtml)
@@ -486,8 +478,6 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						# See The Man Who Was Thursday by G K Chesterton
 						processed_xhtml = regex.sub(r"(<h[1-6] [^>]*) role=\".*?\">", "\\1>", processed_xhtml)
 
-
-
 						# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
 						processed_xhtml = processed_xhtml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
 
@@ -525,19 +515,21 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						processed_xhtml = processed_xhtml.replace(" >", ">")
 						processed_xhtml = regex.sub(r"""\s*epub:type=""\s*""", "", processed_xhtml)
 
-						# Move quotation marks over periods and commas
-						processed_xhtml = regex.sub(fr"([\\.…,])([’”{se.HAIR_SPACE}]+)", r"""\1<span class="quote-align">\2</span>""", processed_xhtml)
+						# We've disabled quote-align for now, because it causes more problems than expected.
+						# # Move quotation marks over periods and commas
+						# # The negative lookahead is to prevent matching `.&hairsp;…`
+						# processed_xhtml = regex.sub(fr"([\\.…,])([’”{se.HAIR_SPACE}]+)(?!…)", r"""\1<span class="quote-align">\2</span>""", processed_xhtml)
 
-						# The above replacement may replace text within <img alt> attributes. Remove those now until no replacements remain, since we may have
-						# many matches in the same line
-						replacements = 1
-						while replacements > 0:
-							processed_xhtml, replacements = regex.subn(r"alt=\"([^<>\"]+?)<span class=\"quote-align\">([^<>\"]+?)</span>", r"""alt="\1\2""", processed_xhtml)
+						# # The above replacement may replace text within <img alt> attributes. Remove those now until no replacements remain, since we may have
+						# # many matches in the same line
+						# replacements = 1
+						# while replacements > 0:
+						# 	processed_xhtml, replacements = regex.subn(r"alt=\"([^<>\"]+?)<span class=\"quote-align\">([^<>\"]+?)</span>", r"""alt="\1\2""", processed_xhtml)
 
-						# Do the same for <title> elements
-						replacements = 1
-						while replacements > 0:
-							processed_xhtml, replacements = regex.subn(r"<title>([^<>]+?)<span class=\"quote-align\">([^<>]+?)</span>", r"""<title>\1\2""", processed_xhtml)
+						# # Do the same for <title> elements
+						# replacements = 1
+						# while replacements > 0:
+						# 	processed_xhtml, replacements = regex.subn(r"<title>([^<>]+?)<span class=\"quote-align\">([^<>]+?)</span>", r"""<title>\1\2""", processed_xhtml)
 
 						if processed_xhtml != xhtml:
 							file.seek(0)
@@ -564,6 +556,11 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							file.write(processed_css)
 							file.truncate()
 
+		# Output the modified content.opf so that we can build the kobo book before making more epub2 compatibility hacks
+		with open(work_epub_root_directory / "epub" / "content.opf", "w", encoding="utf-8") as file:
+			file.write(metadata_xml)
+			file.truncate()
+
 		if build_kobo:
 			with tempfile.TemporaryDirectory() as temp_directory:
 				kobo_work_directory = Path(temp_directory)
@@ -607,9 +604,9 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 							dom = se.easy_xml.EasyXhtmlTree(xhtml)
 
-							# Remove quote-align spans we inserted above, since Kobo has weird spacing problems with them
-							for node in dom.xpath("/html/body//span[contains(@class, 'quote-align')]"):
-								node.unwrap()
+							# # Remove quote-align spans we inserted above, since Kobo has weird spacing problems with them
+							# for node in dom.xpath("/html/body//span[contains(@class, 'quote-align')]"):
+							# 	node.unwrap()
 
 							# Now add the kobo spans
 							kobo.add_kobo_spans_to_node(dom.xpath("/html/body")[0].lxml_element)
@@ -645,6 +642,10 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						processed_css = regex.sub(r"^\s*hyphens\s*:\s*(.+)", "\thyphens: \\1\n\tadobe-hyphenate: \\1\n\t-webkit-hyphens: \\1\n\t-moz-hyphens: \\1", processed_css, flags=regex.MULTILINE)
 						processed_css = regex.sub(r"^\s*hyphens\s*:\s*none;", "\thyphens: none;\n\tadobe-text-layout: optimizeSpeed; /* For Nook */", processed_css, flags=regex.MULTILINE)
 
+						# We add a 20vh margin to sections without heading elements to drop them down on the page a little.
+						# As of 01-2021 the vh unit is not supported on Nook or Kindle (but is on kepub and ibooks).
+						processed_css = regex.sub(r"^(\s*)margin-top\s*:\s*20vh;", r"\1margin-top: 5em;", processed_css, flags=regex.MULTILINE)
+
 						if processed_css != css:
 							file.seek(0)
 							file.write(processed_css)
@@ -811,7 +812,10 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 		# All done, clean the output
 		for filepath in se.get_target_filenames([work_epub_root_directory], (".xhtml", ".svg", ".opf", ".ncx")):
-			se.formatting.format_xml_file(filepath)
+			try:
+				se.formatting.format_xml_file(filepath)
+			except se.SeException as ex:
+				raise se.InvalidXhtmlException(f"{ex}. File: [path][link={filepath}]{filepath}[/][/]") from ex
 
 		# Write the compatible epub
 		se.epub.write_epub(work_epub_root_directory, output_directory / compatible_epub_output_filename)
@@ -896,31 +900,28 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 				with open(work_epub_root_directory / "epub/text/endnotes.xhtml", "r+", encoding="utf-8") as file:
 					xhtml = file.read()
 
-					# We have to remove the default namespace declaration from our document, otherwise
-					# xpath won't find anything at all.  See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 					try:
-						tree = etree.fromstring(str.encode(xhtml.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")))
+						file_dom = se.easy_xml.EasyXhtmlTree(xhtml)
 					except Exception as ex:
 						raise se.InvalidXhtmlException(f"Error parsing XHTML [path][link=file://{(work_epub_root_directory / 'epub/text/endnotes.xhtml').resolve()}]endnotes.xhtml[/][/]. Exception: {ex}")
 
-					notes = tree.xpath("//li[@epub:type=\"endnote\" or @epub:type=\"footnote\"]", namespaces=se.XHTML_NAMESPACES)
-
 					processed_endnotes = ""
 
-					for note in notes:
-						note_id = note.get("id")
+					# Loop over each endnote and move the ending backlink to the front of the endnote for Kindles
+					for note in file_dom.xpath("//li[@epub:type='endnote' or @epub:type='footnote']"):
+						note_id = note.get_attr("id")
 						note_number = note_id.replace("note-", "")
 
 						# First, fixup the reference link for this endnote
 						try:
-							ref_link = etree.tostring(note.xpath("p[last()]/a[last()]")[0], encoding="unicode", pretty_print=True, with_tail=False).replace(" xmlns:epub=\"http://www.idpf.org/2007/ops\"", "").strip()
+							ref_link = note.xpath("p[last()]/a[last()]")[0].to_string()
 						except Exception as ex:
 							raise se.InvalidXhtmlException(f"Can’t find ref link for [url]#{note_id}[/].") from ex
 
 						new_ref_link = regex.sub(r">.*?</a>", ">" + note_number + "</a>.", ref_link)
 
 						# Now remove the wrapping li node from the note
-						note_text = regex.sub(r"^<li[^>]*?>(.*)</li>$", r"\1", etree.tostring(note, encoding="unicode", pretty_print=True, with_tail=False), flags=regex.IGNORECASE | regex.DOTALL)
+						note_text = regex.sub(r"^<li[^>]*?>(.*)</li>$", r"\1", note.to_string(), flags=regex.IGNORECASE | regex.DOTALL)
 
 						# Insert our new ref link
 						result = regex.subn(r"^\s*<p([^>]*?)>", "<p\\1 id=\"" + note_id + "\">" + new_ref_link + " ", note_text)
